﻿// ©Copyright 2011 McGee Systems Inc. (dba NetbizSystems.com)
using System;
using System.Diagnostics;
using System.Linq;
using Netbiz.Bizworks.Data;
using NLog;

namespace Netbiz.Bizworks
{
    /// <summary>
    /// Bizworks[BaseApplicationManager] Class
    /// </summary>
    public abstract partial class BaseApplicationManager
    {
        /// <summary>
        /// BaseApplicationManager Constructor
        /// </summary>
        /// <param name="logger"></param>
        /// <param name="fileManager"></param>
        /// <param name="platform"></param>
        public BaseApplicationManager(NLog.Logger logger, BaseFileManager fileManager, BaseSystemManager platform)
        {
            BaseSystemManager = platform;
            Logger = logger;
            FileManager = fileManager;
        }

        /// <summary>
        /// don't allow this ;)
        /// </summary>
        private BaseApplicationManager() { }

        /// <summary>
        /// reference to the runtime platform that we are in (wpf,asp,wcf,console)
        /// </summary>
        public BaseSystemManager BaseSystemManager { get; private set; }
        public ApplicationSystemType SystemType { get { return BaseSystemManager.ApplicationSystemType; } }

        #region NETBIZ Global Helpers
        public Logger Logger { get; private set; }
        public BaseFileManager FileManager { get; private set; }
        #endregion

        private static readonly TraceSource _bwTraceSource = new TraceSource("BizworksLog");
        private const string _msg1 = "Bizworks application created.";
        private ApplicationProfile _bwProfile;

        /// <summary>
        /// GetUsersList that derived class call when constructor is complete
        /// </summary>
        protected void InializeClass(ApplicationProfile pProfile)
        {
            _bwProfile = pProfile;
            this.ApplicationStarting();
        }

        private void ApplicationStarting()
        {
            this.LogEvent(_msg1);
        }
        public void ApplicationEnding()
        {
            //this.LogEvent("ending");
        }

        #region Public Methods
        /// <summary>
        /// Application-wide event logger
        /// </summary>
        /// <param name="logText"></param>
        /// <param name="tet"></param>
        public void LogEvent(string logText)
        {
            // format the event.log message the way we like
            var xyz = this._bwProfile.ApplicationName + "[" + BaseSystemManager.ApplicationSystemType + "]" + " - " + logText;

            Logger.Info(xyz);
        }

        /// <summary>
        /// Application-wide exception logger
        /// </summary>
        /// <param name="exp"></param>
        public void LogException(Exception exp)
        {
            Logger.InfoException("hello dave",exp);
        }

        /// <summary>
        /// Get Bizworks page profile
        /// </summary>
        /// <param name="pageId">Id of requested page</param>
        /// <returns></returns>
        public ApplicationProfile GetApplicationProfile()
        {
            return _bwProfile;
        }

        /// <summary>
        /// get a liveCongig value from the bizworks (sql) schema
        /// </summary>
        /// <param name="pType"></param>
        /// <param name="pId"></param>
        /// <param name="pVariableName"></param>
        /// <returns></returns>
        public string GetLiveConfigValue(int pType, string pId, string pVariableName)
        {
            using (Data_NetBizDataContext db = new Data_NetBizDataContext())
            {
                var row = (from q in db.LiveConfig
                           where q.dovId == pType && q.lpvId == pId && q.lpvVariableName == pVariableName
                           select q).FirstOrDefault();

                if (row != null)
                {
                    return row.lpvVariableValue.Trim();
                }
                else
                {
                    return null;
                }
            }
        }

        public string GetMessageText(string pPk)
        {
            var dataModel = new BizworksConfigModel();
            var messageXmlRepository = dataModel.GetRepository<XmlMessageRepository, XmlMessage>();
            var result = messageXmlRepository.GetByPk(pPk);
            if (result != null) return result.MessageText;

            return "message (" + pPk + ") not found in XmlMessageRepository.";
        }
        #endregion
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Bizworks[ApplicationProfile] structure
    /// </summary>
    public struct ApplicationProfile
    {
        public string ApplicationName;
        public string ClassName;
        public string ContainerName;
        public string ApplicationVersion;
    }
}
