﻿// ©Copyright 2011 McGee Systems Inc. (dba NetbizSystems.com)
using System.Collections.Generic;

namespace Netbiz.Bizworks
{
    /// <summary>
    /// Bizworks[BaseSystemManager] class
    /// </summary>
    public abstract class BaseSystemManager
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="ast">Application/platform reference</param>
        protected BaseSystemManager(ApplicationSystemType ast)
        {
            ApplicationSystemType = ast;
        }

        /// <summary>
        /// Enum that reveals the type (platform) of this system
        /// </summary>
        public ApplicationSystemType ApplicationSystemType { get; private set; }
        public string ApplicationSystemsName { get; protected set; }

        private readonly IList<PageProfile> _pages = new List<PageProfile>();

        #region Public Methods
        /// <summary>
        /// Get Bizworks page profile
        /// </summary>
        /// <param name="pageId">Id of requested page</param>
        /// <returns></returns>
        public PageProfile GetPageProfile(string pageId)
        {
            foreach (var page in _pages)
            {
                if (page.PageName.ToUpper() == pageId.ToUpper())
                return page;
            }
            return new PageProfile(); // null when not found
        }
        /// <summary>
        /// Called by generated code in derived class
        /// </summary>
        /// <param name="page"></param>
        protected void SetPageProfile(PageProfile page)
        {
            _pages.Add(page);
        }
        #endregion
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Structure that defines a Bizworks system page
    /// </summary>
    public struct PageProfile
    {
        public string PageName;
        public string PageTitle;
        public string PageHelpFile;
        public string PageAspx;
        public string PageScript;
        public string PageService;
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Bizworks[WpfSystemManager] class 
    /// </summary>
    public class WpfSystemManager : BaseSystemManager
    {
        public WpfSystemManager() : base(ApplicationSystemType.Wpf)
        {
            //
        }
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Bizworks[WcfSystemManager] class
    /// </summary>
    public class WcfSystemManager : BaseSystemManager
    {
        public WcfSystemManager() : base(ApplicationSystemType.Wcf)
        {
            //
        }
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Bizworks[AspSystemManager] class
    /// </summary>
    public class AspSystemManager : BaseSystemManager
    {
        public string ScriptName { get; protected set; }

        public AspSystemManager() : base(ApplicationSystemType.Asp)
        {
            //
        }
        public virtual string GetApplicationScript()
        {
            return ScriptName;
        }
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// Bizworks[Service] class
    /// </summary>
    public class ServiceSystemManager : BaseSystemManager
    {
        public ServiceSystemManager() : base(ApplicationSystemType.Service)
        {
            //
        }
    }

    // //////////////////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// 
    /// </summary>
    public enum ApplicationSystemType
    {
        Wpf     = 1,
        Asp     = 2,
        Wcf     = 3,
        Console = 4,
        Service = 5
    }
}
