﻿// ©Copyright 2011 McGee Systems Inc. (dba NetbizSystems.com)
using System;
using System.Collections.Generic;
using System.Web;
using Netbiz.Bizworks.Data;
using Netbiz.Bizworks.Wcf;
using Netbiz.Bizworks.Web;

namespace Netbiz.Bizworks
{
    /// <summary>
    /// Bizworks[BasePresenter] class - serves both ASP and WPF projects 
    /// </summary>
    /// <remarks>
    /// Keep in mind that this presenter needs to work in an ASP application
    /// AND/OR a WPF application, or Wcf-service.
    /// </remarks>
    public abstract class BasePresenter
    {
        /// <summary>
        /// Bizworks[BasePresenter] constructor 
        /// </summary>
        protected BasePresenter(BaseIocContainer pBaseIocContainer = null)
        {
            if (pBaseIocContainer != null)
            {
                BwContainer = pBaseIocContainer;
            }
            else
            {
                // must be a wcf service.. go find the container
                BwContainer = (HttpContext.Current.ApplicationInstance.GetBizworksContainer());
            }
        }

        #region BINDABLE PROPERTIES
        public string PageServiceFile = "Services/HomePage.svc";
        #endregion

        /// <summary>
        /// 
        /// </summary>
        public void ResetEvents()
        {
            this.RaiseNavigationEvent = null;
        }
        /// <summary>
        /// get dropdown values.
        /// </summary>
        /// <param name="pDropDownId"></param>
        /// <returns>list of UiDropDownItem items</returns>
        /// <remarks>
        /// gets called from basePage when rendering UiItem(s).  if derived presenter
        /// does not override this method, the dropdown will be empty.
        /// </remarks>
        public virtual List<UiDropDownItem> GetDropDownList(string pDropDownId)
        {
            return new List<UiDropDownItem>();
        }
        /// <summary>
        /// Reference to IocContainer
        /// </summary>
        protected BaseIocContainer BwContainer { get; private set; }

        #region - INTERFACE (IBaseX) MEMBERS
        /// <summary>
        /// 
        /// </summary>
        /// <param name="dc"></param>
        /// <param name="returnCode"></param>
        /// <param name="success"></param>
        /// <returns></returns>
        public BaseDataContract SetReturnDataContract(BaseDataContract dc, int returnCode, Boolean success = true)
        {
            dc.ResultCode = returnCode;
            dc.Success = success;
            dc.LoggingId = System.Guid.NewGuid().ToString();

            BwContainer.ApplicationManager.LogEvent(success
                ? "service completed successfully"
                : "service completed with error(s)"
            );

            return dc;
        }
        #endregion

        #region PRESENTER EVENT/HANDLER
        // http://msdn.microsoft.com/en-us/library/w369ty8x.aspx

        /// <summary>
        /// Application/UCase navigation request (event)
        /// </summary>
        public event EventHandler<NavigatonEventArgs> RaiseNavigationEvent;

        /// <summary>
        /// allow derived classes to override the event invocation behavior
        /// </summary>
        /// <param name="e"></param>
        protected virtual void OnRaiseNavigationEvent(NavigatonEventArgs e)
        {
            var handler = RaiseNavigationEvent;

            if (handler != null)
            {
                handler(this, e);
            }
        }
        #endregion
    }

// ////////////////////////////////////////////////////////////////////////////////////////////CLASS

    public class NavigatonEventArgs : EventArgs
    {
        public NavigatonEventArgs(string s)
        {
            _pageAspx = s;
        }
        private readonly string _pageAspx;
        public string PageAspx
        {
            get { return _pageAspx; }
        }
    }
}
