﻿using System.Configuration;
using System.Data.Objects;
using System.Data.Objects.DataClasses;

namespace Netbiz.Bizworks.Data
{
    public abstract class BaseModel
    {
        protected BaseModel() { return; }
    }

    // ///////////////////////////////////////////////////////////////////////////////////////CLASS

    public class ModelForEntityFramework<TEntities> : BaseModel where TEntities : ObjectContext, new()
    {
        private readonly TEntities _modelEntities = null;
        public TEntities ModelEntities
        {
            get { return _modelEntities ?? this.GetNewContext(); }
        }

        /// <summary>
        /// reference to a new EfModel context
        /// </summary>
        /// <returns>context for this model</returns>
        public TEntities GetNewContext()
        {
            return new TEntities();
        }

        public virtual TRepository GetRepository<TRepository, TEntityObject, TEntityDto>(TEntities pObjectEntities = null)
            where TRepository : BaseRepositoryForEntityFramework<TEntities, TEntityObject, TEntityDto>, new()
            where TEntityObject : EntityObject
            where TEntityDto : BaseDto
        {
            return new TRepository { DbContext = pObjectEntities ?? this.ModelEntities };
        }
    }

    // ///////////////////////////////////////////////////////////////////////////////////////CLASS

    public class ModelForXml : BaseModel
    {
        public ModelForXml(string pConnectionSetting)
        {
            XmlFolder = ConfigurationManager.ConnectionStrings[pConnectionSetting].ConnectionString;
        }

        public string XmlFolder { get; private set; }
        public TRepository GetRepository<TRepository,TDto>()
            where TRepository : BaseRepositoryForXml<TDto>, new()
            where TDto : BaseDtoForXml, new()
        {
            return new TRepository() { XmlFolder = XmlFolder };
        }
    }
}
