﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Data.Objects;
using System.Data.Objects.DataClasses;
using System.IO;
using System.Linq;
using System.Linq.Expressions;
using System.Xml;
using System.Xml.Serialization;

namespace Netbiz.Bizworks.Data
{
    public abstract class BaseRepository { }

    // ///////////////////////////////////////////////////////////////////////////////////////CLASS

    /// <summary>
    /// 
    /// </summary>
    /// <typeparam name="TContext"></typeparam>
    /// <typeparam name="TEntity"></typeparam>
    /// <typeparam name="TEntityDto"></typeparam>
    public abstract class BaseRepositoryForEntityFramework<TContext, TEntity, TEntityDto> : BaseRepository
        where TContext : ObjectContext
        where TEntity : EntityObject
        where TEntityDto : BaseDto
    {
        protected BaseRepositoryForEntityFramework() { return; }

        // TODO: leave this public for now
        public TContext DbContext = default(TContext);

        public abstract IQueryable<TEntity> LoadAll();
        public abstract IQueryable<TEntityDto> LoadAllDto();
        public abstract void New(TEntity pEntity, bool pCommitChanges = false);
        public abstract TEntity LoadByKey(IEnumerable<KeyValuePair<string, object>> pKeyValues);
        public abstract IQueryable<TEntity> Query(Expression<Func<TEntity, bool>> filter);

        public XmlDocument ToXml(TEntity pEntity)
        {
            var xmlDoc = new XmlDocument();
            var xmlSerializer = new XmlSerializer(typeof(TEntity));
            using (var memoryStream = new MemoryStream())
            {
                xmlSerializer.Serialize(memoryStream, pEntity);
                memoryStream.Position = 0;
                xmlDoc.Load(memoryStream);
                return xmlDoc;
            }
        }
    }

    // ///////////////////////////////////////////////////////////////////////////////////////CLASS

    /// <summary>
    /// xml repository that allows for access to xml entities
    /// </summary>
    /// <typeparam name="TDto">nade mapping class</typeparam>
    public class BaseRepositoryForXml<TDto> where TDto : BaseDtoForXml, new()
    {
        public BaseRepositoryForXml() { }

        /// <summary>
        /// file directory where files are stored
        /// </summary>
        public string XmlFolder { get; set; }
        /// <summary>
        /// xml file name
        /// </summary>
        public string XmlFile { get; set; }
        /// <summary>
        /// xpath to a single row in the file
        /// </summary>
        public string XmlPath { get; set; }
        /// <summary>
        /// name of attribute that represents the pk
        /// </summary>
        public string EntityPk { get; set; }
        /// <summary>
        /// container for all rows
        /// </summary>
        private XmlDocument _xmlDocument = null;
        private XmlDocument XmlDocument
        {
            get
            {
                if (_xmlDocument == null)
                {
                    _xmlDocument = new XmlDocument();
                    _xmlDocument.Load(XmlFolder + "\\" + XmlFile);
                }
                return _xmlDocument;
            }
        }
        /// <summary>
        /// get all rows from xml entity
        /// </summary>
        /// <returns></returns>
        public IList<TDto> GetAll()
        {
            var rtn = new List<TDto>();
            foreach(XmlNode xmlNode in XmlDocument.SelectNodes(XmlPath))
            {
                rtn.Add(GetDto(xmlNode));
            }
            return rtn;
        }
        /// <summary>
        /// 
        /// </summary>
        /// <param name="pAttribute"></param>
        /// <param name="pValue"></param>
        /// <returns></returns>
        public IList<TDto> GetAll(string pAttribute, string pValue)
        {
            var xpath = string.Format("{0}[@{1}='{2}']", XmlPath, pAttribute, pValue);
            var rtn = new List<TDto>();
            foreach (XmlNode xmlNode in XmlDocument.SelectNodes(XmlPath))
            {
                rtn.Add(GetDto(xmlNode));
            }
            return rtn;
        }
        /// <summary>
        /// get a single row from an xml entity
        /// </summary>
        /// <param name="pPkValue"></param>
        /// <returns></returns>
        public TDto GetByPk(string pPkValue)
        {
            var xpath = string.Format("{0}[@{1}='{2}']", XmlPath, EntityPk, pPkValue);
            var node = XmlDocument.SelectSingleNode(xpath);
            if (node != null)
            {
                return this.GetDto(node);
            }
            return null;
        }

        /// <summary>
        /// get a fully mapped xml entity
        /// </summary>
        /// <param name="pNode"></param>
        /// <returns></returns>
        private TDto GetDto(XmlNode pNode)
        {
            return new TDto() { XmlNode = pNode } as TDto;
        }
    }
}
