﻿using System;
using System.Web.UI.WebControls;
using System.Reflection;

namespace Netbiz.Bizworks
{
    public class PresenterDataSource : ObjectDataSource
    {
        /// <summary>
        /// ObjectDataSource from CaseManager
        /// </summary>
        /// <remarks>
        /// This extension of the standard ObjectDataSource is used on the page to wireup data from the 
        /// CaseManager that is associated with the page (by convention).
        /// </remarks>
        public PresenterDataSource()
        {
            this.ObjectCreating += new ObjectDataSourceObjectEventHandler(DataSourceObjectCreating);
            this.ObjectDisposing += new ObjectDataSourceDisposingEventHandler(DataSourceObjectDisposing);

            return;
        }

        /// <summary>
        /// DataSource Disposing
        /// </summary>
        /// <remarks>
        /// Since we created our own DataObject and since it might be used by several ODSs, don't dispose of it yet.
        /// </remarks>
        static void DataSourceObjectDisposing(object sender, ObjectDataSourceDisposingEventArgs e)
        {
            e.Cancel = true; // do NOT displose of the caseManager
        }

        /// <summary>
        /// CaseDataSource Creating
        /// </summary>
        /// <remarks>
        /// The ODS is about to create the type associated with itself.  Since we are using the CaseManager as the object, swap 
        /// in that object thus preventing the type from beiing created.
        /// </remarks>
        void DataSourceObjectCreating(object sender, ObjectDataSourceEventArgs e)
        {
            // PagePresenter is a PUBLIC property on the base page
            Type type = Page.GetType();
            PropertyInfo pi = type.GetProperty("PagePresenter");
            object oCase = pi.GetValue(Page, null);

            // use the PagePresenter as the object for this ods
            e.ObjectInstance = oCase;
        }
    }
}

