﻿// ©Copyright 2011 McGee Systems Inc. (dba NetbizSystems.com)
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Xml.Linq;

namespace Netbiz.Bizworks
{
    public class TtHelpers
    {
        /// <summary>
        /// Extract the model/xml from the passed in assembly
        /// </summary>
        /// <param name="ass"></param>
        /// <param name="res"></param>
        /// <returns>XDocument</returns>
        public XDocument GetXmlModel(Assembly ass, string res)
        {
            using (var resourceStream = ass.GetManifestResourceStream(res))
            {
                return XDocument.Load(resourceStream);
            }
        }

        /// <summary>
        /// return a list of domainValue rows
        /// </summary>
        /// <param name="domName"></param>
        /// <returns></returns>
        public IList<DomainValues> GetDomainValues(string domName)
        {
            IList<DomainValues> doms = null;
            using (var db = new Data_NetBizDataContext())
            {
                doms = (from rows in db.DomainValues where rows.Domain.domName == domName select rows).ToList();
            }
            return doms;
        }

        /// <summary>
        /// return a list of domain rows
        /// </summary>
        /// <returns></returns>
        public IList<Domain> GetDomainNames()
        {
            IList<Domain> doms = null;
            using (var db = new Data_NetBizDataContext())
            {
                doms = (from rows in db.Domain select rows).ToList();
            }
            return doms;
        }

        /// <summary>
        /// return a list of (string) LiveConfig names
        /// </summary>
        /// <param name="pType"></param>
        /// <param name="pId"></param>
        /// <returns></returns>
        public IList<string> GetLiveConfigNames(int pType, string pId)
        {
            using (var db = new Data_NetBizDataContext())
            {
                var rows = from lpv in db.LiveConfig
                            where lpv.dovId == pType && lpv.lpvId == pId
                            select lpv.lpvVariableName.Trim();

                return rows.ToList();
            }
        }
    }
}
