﻿using System;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.Web;
using Netbiz.Bizworks.Web;

namespace Netbiz.Bizworks.Wcf
{
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public abstract class BaseService<TApplicationManager> where TApplicationManager : BaseApplicationManager
    {
        /// <summary>
        /// normal constructor called by Wcf
        /// </summary>
        public BaseService() : this(HttpContext.Current.ApplicationInstance.GetBizworksContainer())
        {
            var occ = OperationContext.Current;
        }
        /// <summary>
        /// constructor to allows for unit testing
        /// </summary>
        /// <param name="pContainer"></param>
        /// <remarks>leave public for use by unitest</remarks>
        public BaseService(BaseIocContainer pContainer)
        {
            BwContainer = pContainer;
            BizworksApplicationManager = (TApplicationManager)BwContainer.Resolve(typeof(TApplicationManager));
        }

        /// <summary>
        /// Bizworks IOC Container
        /// </summary>
        public BaseIocContainer BwContainer { get; private set; }

        /// <summary>
        /// Bizworks ApplicationManager
        /// </summary>
        public TApplicationManager BizworksApplicationManager { get; private set; }

        /// <summary>
        /// 
        /// </summary>
        public string ClientUserAgent
        {
            get {
                if (System.ServiceModel.Web.WebOperationContext.Current != null)
                    return System.ServiceModel.Web.WebOperationContext.Current.IncomingRequest.Headers["User-Agent"];
                return string.Empty;
            }
        }

        /// <summary>
        /// ALL services use this method to finalize the datacontract they are returning
        /// </summary>
        /// <param name="dc"></param>
        /// <param name="returnCode"></param>
        /// <param name="success"></param>
        /// <returns></returns>
        protected BaseDataContract SetReturnDataContract(BaseDataContract dc, int returnCode, Boolean success = true)
        {
            dc.ResultCode = returnCode;
            dc.Success = success;
            dc.LoggingId = System.Guid.NewGuid().ToString();

            BizworksApplicationManager.LogEvent(success
                ? "service completed successfully"
                : "service completed with error(s)",
                System.Diagnostics.TraceEventType.Information
            );

            return dc;
        }
    }
}
