﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Linq;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using Netbiz.Bizworks.Data;
using Netbiz.Bizworks.Wcf;
using Netbiz.Sample.DataAccess;
using Netbiz.Sample.Application.DatabaseModels;

namespace Netbiz.Sample.Application.Presenters
{
    /// <summary>
    /// CustomerHomePresenter presenter class... another part(ial) has been generated
    /// </summary>
    /// <remarks>This presenter/class serves both the code-behind and the page.js</remarks>
    [DataObject]
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public partial class CustomerHomePresenter : ICustomerHomeService
    {
        public CustomerHomePresenter() : this(null)
        {
            //wcf services need this parameterless constructor           
        }
        /// <summary>
        /// this is where most/all of the biz logic for this presenter/service resides
        /// </summary>
        private static CustomerCaseManager1 _customerCaseManager1;
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public IList<Customer> GetCustomerList()
        {
            var repository = this.AdventureWorksApplicationManager.AdventureWorksModel.GetRepository<CustomerRepository, Customer, CustomerDto>();
            var query = repository.LoadAll().OrderBy(rows => rows.LastName);
            return query.ToList();
        }
        /// <summary>
        /// 
        /// </summary>
        /// <returns></returns>
        public IList<SalesOrderHeader> GetSalesOrderList(int pCustomerId)
        {
            var salesRepository = this.AdventureWorksApplicationManager.AdventureWorksModel.GetRepository<SalesOrderHeaderRepository, SalesOrderHeader, SalesOrderHeaderDto>();
            var salesPkValues = new KeyValuePair<string, object>[] { new KeyValuePair<string, object>("CustomerID", 123) };
            return salesRepository.Query(x => x.CustomerID == pCustomerId).ToList();
        }
        /// <summary>
        /// return DataItem(s) based on UseCase configutation
        /// </summary>
        /// <param name="pCustomerId"></param>
        /// <returns>IList.UiDataItem</returns>
        public IList<UiDataItem> GetCustomerItems(int pCustomerId)
        {
            var rtn = new List<UiDataItem>();

            var customerRepository = this.AdventureWorksApplicationManager.AdventureWorksModel.GetRepository<CustomerRepository, Customer, CustomerDto>();
            var customerPkValues = new KeyValuePair<string, object>[] { new KeyValuePair<string, object>("CustomerID", pCustomerId) };
            var customerRow = customerRepository.LoadByKey(customerPkValues);

            var useCaseDataItemRepository = this.AdventureWorksApplicationManager.AdventureWorksXmlModel.GetRepository<UseCaseDataItemsXmlRepository, UseCaseDataItemsXml>();
            var useCaseDataItemRows = useCaseDataItemRepository.GetAll("Id", "usecase1");
            foreach (var ucdi in useCaseDataItemRows)
            {
                int displayLength;
                int.TryParse(ucdi.DataDisplayLength,out displayLength);
                var itemValue = string.Empty;
                var isRequired = bool.Parse(ucdi.IsRequired);
                switch (ucdi.DataItemName)
                {
                    case "Title": itemValue = customerRow.Title ?? ""; break;
                    case "LastName": itemValue = customerRow.LastName; break;
                    case "FirstName": itemValue = customerRow.FirstName; break;
                    case "EmailAddress": itemValue = customerRow.EmailAddress; break;
                    case "Custom1": itemValue = "1"; break;
                    default: itemValue = "unknown-item"; break;
                }
                rtn.Add(new UiDataItem()
                            {
                                DataName = ucdi.DataItemName,
                                DataLabel = ucdi.DataLabel,
                                DataValue = itemValue,
                                DataDisplayLength = displayLength,
                                DataControl = ucdi.DataControl,
                                IsRequired = isRequired,
                                RegEx = ucdi.RegEx
                            });
            }                
            return rtn;
        }
        /// <summary>
        /// get dropdown values.
        /// </summary>
        /// <param name="pRequest"></param>
        /// <returns>list of UiDropDownItem(s)</returns>
        /// <remarks>
        /// gets called from basePage while rendering UiItem(s) when one is marked as
        /// a dropdown type item.
        /// </remarks>
        public override List<UiDropDownItem> GetDropDownList(string pRequest)
        {
            var rtn = base.GetDropDownList(pRequest);
            switch (pRequest)
            {
                case "Title" :
                    rtn.Add(new UiDropDownItem() {DataText = "Mr.", DataValue = "Mr."});
                    rtn.Add(new UiDropDownItem() { DataText = "Ms.", DataValue = "Ms." });
                    rtn.Add(new UiDropDownItem() { DataText = "", DataValue = "" });
                    break;
                case "Custom1":
                    rtn.Add(new UiDropDownItem() { DataText = "1", DataValue = "1" });
                    break;

                default: break;
            }
            return rtn;
        }

        #region - WCF INTERFACE MEMBERS/operations
        public EmptyDataContract GetEmptyData(EmptyDataContract composite)
        {
            _customerCaseManager1 = (CustomerCaseManager1)base.BwContainer.Resolve(typeof(CustomerCaseManager1));

            var emptyDataContract = new EmptyDataContract();
            try
            {
                var rtn = _customerCaseManager1.DoSomething(AuthMethodEnum.Other);
            }
            catch (Exception)
            {
                {
                    return (EmptyDataContract)base.SetReturnDataContract(emptyDataContract, 999, false);
                }
                throw;
            }
            return (EmptyDataContract)base.SetReturnDataContract(emptyDataContract, 1);
        }
        #endregion
    }

    /// <summary>
    /// ServiceContract (interface) for the CustomerHomePresenter
    /// </summary>
    [ServiceContract(Namespace = "AdventureWorks.Wcf.CustomerHomeService")]
    public partial interface ICustomerHomeService
    {
        [OperationContract, WebGet(ResponseFormat = WebMessageFormat.Json)]
        EmptyDataContract GetEmptyData(EmptyDataContract composite);
    }
}

