﻿using System;
using System.Runtime.Serialization;
using System.ServiceModel;
using System.ServiceModel.Activation;
using System.ServiceModel.Web;
using Netbiz.Bizworks;
using Netbiz.Bizworks.Wcf;
using Netbiz.Sample.Application.DataContracts;

namespace Netbiz.Sample.Application.Presenters
{
    /// <summary>
    /// HomePagePresenter presenter/class 
    /// </summary>
    /// <remarks>This presenter/class serves both the code-behind and the page.js</remarks>
    [AspNetCompatibilityRequirements(RequirementsMode = AspNetCompatibilityRequirementsMode.Allowed)]
    public partial class HomePagePresenter : IHomePageService
    {
        /// <summary>
        /// 
        /// </summary>
        public HomePagePresenter() : this(null)
        {
            //wcf services need this parameterless constructor           
        }
        /// <summary>
        /// this is where most/all of the biz logic for this presenter/service resides
        /// </summary>
        private static CustomerCaseManager1 _customerCaseManager1;
        /// <summary>
        /// sample presenter method
        /// </summary>
        public EmptyDataContract DoSomething()
        {
            // You have reference to the top-level application manager
            var rtnApp = this.AdventureWorksApplicationContainer.AdventureWorksApplicationManager.DoSomething(AuthMethodEnum.Forms);

            // Or, any of the middle-level subject managers
            var rtnSubject = this.AdventureWorksApplicationContainer.CustomerSubjectManager.DoSomething(AuthMethodEnum.Other);

            // Or, any of the bottom-level case managers)

            return (EmptyDataContract)base.SetReturnDataContract(rtnApp, 1);
        }
        /// <summary>
        /// determine .aspx based on subjectId
        /// </summary>
        /// <param name="pSubjectId">subject id string from markup</param>
        public void ShowApplicationHome(string pSubjectId)
        {
            string pageName = string.Empty;
            switch (pSubjectId.ToUpper())
            {
                case CUSTOMER_SUBJECT_MANAGER:
                    pageName = AdventureWorksApplicationContainer.CustomerSubjectManager.HomePage;
                    break;
                case PRODUCT_SUBJECT_MANAGER:
                    pageName = AdventureWorksApplicationContainer.ProductSubjectManager.HomePage;
                    break;
                case ADMIN_MANAGER:
                    pageName = AdventureWorksApplicationContainer.AdventureWorksAdminManager.HomePage;
                    break;
                default:
                    break;
            }
            var pageProfile = AdventureWorksApplicationContainer.SystemManager.GetPageProfile(pageName);

            // TODO: temp code to test event (dka)
            OnRaiseNavigationEvent(new NavigatonEventArgs(pageProfile.PageAspx));
        }

        private const string CUSTOMER_SUBJECT_MANAGER = "CUSTOMERSUBJECTMANAGER";
        private const string PRODUCT_SUBJECT_MANAGER = "PRODUCTSUBJECTMANAGER";
        private const string ADMIN_MANAGER = "ADMINMANAGER";

        #region BINDING PROPERTIES
        public string GreetingText
        {
            get
            {
                return this.AdventureWorksApplicationContainer.AdventureWorksApplicationManager.GetMessageText("xyz");
            }
        }
        #endregion

        #region WCF INTERFACE MEMBERS/operations
        public EmptyDataContract GetEmptyData(EmptyDataContract composite)
        {
            _customerCaseManager1 = (CustomerCaseManager1)base.BwContainer.Resolve(typeof(CustomerCaseManager1));

            var emptyDataContract = new EmptyDataContract();
            try
            {
                var rtn = _customerCaseManager1.DoSomething(AuthMethodEnum.Other);
            }
            catch (Exception)
            {
                {
                    return (EmptyDataContract)base.SetReturnDataContract(emptyDataContract, 999, false);
                }
                throw;
            }
            return (EmptyDataContract)base.SetReturnDataContract(emptyDataContract, 1);
        }
        public ListDataContract<MovieData> GetMovieData(int value)
        {
            var returnData = new ListDataContract<MovieData>();

            returnData.ListValue.Add(new MovieData() { Name = "Packers", ReleaseYear = "2011" });
            returnData.ListValue.Add(new MovieData() { Name = "Saints", ReleaseYear = "2010" });
            returnData.ListValue.Add(new MovieData() { Name = "Steelers", ReleaseYear = "2009" });
            returnData.ListValue.Add(new MovieData() { Name = "Steelers", ReleaseYear = "2009" });
            returnData.ListValue.Add(new MovieData() { Name = "Steelers", ReleaseYear = "2009" });
            returnData.ListValue.Add(new MovieData() { Name = "Giants", ReleaseYear = "2008" });
            returnData.ListValue.Add(new MovieData() { Name = "Colts", ReleaseYear = "2007" });
            returnData.ListValue.Add(new MovieData() { Name = "Steelers", ReleaseYear = "2006" });
            returnData.ListValue.Add(new MovieData() { Name = "Patriots", ReleaseYear = "2005" });
            returnData.ListValue.Add(new MovieData() { Name = "Patriots", ReleaseYear = "2004" });
            returnData.ListValue.Add(new MovieData() { Name = "Buccaneers", ReleaseYear = "2003" });
            returnData.ListValue.Add(new MovieData() { Name = "Patriots", ReleaseYear = "2002" });

            return returnData;
        }
        #endregion
    }

// ////////////////////////////////////////////////////////////////////////////////////////////////

    /// <summary>
    /// ServiceContract for the HomePagePresenter
    /// </summary>
    [ServiceContract(Namespace = "AdventureWorks.Wcf.HomePageService")]
    public partial interface IHomePageService
    {
        [OperationContract, WebGet(ResponseFormat = WebMessageFormat.Json)]
        EmptyDataContract GetEmptyData(EmptyDataContract composite);

        [OperationContract, WebGet(ResponseFormat = WebMessageFormat.Json)]
        ListDataContract<MovieData> GetMovieData(int value);
    }
}

// ////////////////////////////////////////////////////////////////////////////////////////////////

namespace Netbiz.Sample.Application.DataContracts
{
    [DataContract]
    public class MovieData
    {
        [DataMember]
        public string Name { get; set; }
        [DataMember]
        public string ReleaseYear { get; set; }
    }
}