﻿// ©Copyright 2011 McGee Systems Inc. (dba NetbizSystems.com)
using System;
using System.Diagnostics;
using Munq;
using Munq.LifetimeManagers;
using Netbiz.Sample.Application.Presenters;
using Netbiz.Sample.Application;
using Netbiz.Sample.Application.Projects;

namespace Netbiz.Sample.WebApplication
{
    public class Global : Netbiz.Bizworks.Web.BaseGlobal<AdventureWorksApplicationContainer>
    {
        /// <summary>
        /// 
        /// </summary>
        public Global() : base()
        {
            if (BizworksContainer == null)
            {
                AdventureWorksAspProject = new AdventureWorksAspProject(); // container supports many/any platform type
                BizworksContainer = new AdventureWorksApplicationContainer(AdventureWorksAspProject);
                ILifetimeManager lifetime = new Munq.LifetimeManagers.ContainerLifetime();
                BizworksContainer.UsesDefaultLifetimeManagerOf(lifetime);

                // register any/all application specific classes in the global Ioc.container
                BizworksContainer.Register<HomePagePresenter>(c => new HomePagePresenter(BizworksContainer));
                BizworksContainer.Register<AdminHomePresenter>(c => new AdminHomePresenter(BizworksContainer));
                BizworksContainer.Register<ReportPresenter>(c => new ReportPresenter(BizworksContainer));
                BizworksContainer.Register<CustomerHomePresenter>(c => new CustomerHomePresenter(BizworksContainer));
                BizworksContainer.Register<ProductHomePresenter>(c => new ProductHomePresenter(BizworksContainer));
                BizworksContainer.Register<EmptyPresenter>(c => new EmptyPresenter(BizworksContainer));
                
                BizworksContainer.AdventureWorksApplicationManager.LogEvent(Msg1);
            }

            return;
        }

        /// <summary>
        /// 
        /// </summary>
        public static AdventureWorksAspProject AdventureWorksAspProject { get; private set; }

        private const string Msg1 = "global application is starting";
        private const string Msg2 = "global application is ending";
        private const string Msg3 = "global application error";

        /// <summary>
        /// this fires many times for each page request?
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Application_BeginRequest(object sender, EventArgs e)
        {
            return;
        }

        /// <summary>
        /// Code that runs on application startup
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Application_Start(object sender, EventArgs e)
        {
            //Exception lastException = Server.GetLastError(); 
            NLog.Logger logger = NLog.LogManager.GetCurrentClassLogger(); 
            logger.Info("A");
        }

        /// <summary>
        /// Code that runs on application shutdown
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Application_End(object sender, EventArgs e)
        {
            BizworksContainer.AdventureWorksApplicationManager.LogEvent(Msg2); 
        }

        /// <summary>
        /// Code that runs when an unhandled error occurs
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Application_Error(object sender, EventArgs e)
        {
            BizworksContainer.AdventureWorksApplicationManager.LogEvent(Msg3); 
        }

        /// <summary>
        /// Code that runs when a new session is started
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Session_Start(object sender, EventArgs e)
        {
            // 
        }

        /// <summary>
        /// Code that runs when a session ends. 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        void Session_End(object sender, EventArgs e)
        {
            // Note: The Session_End event is raised only when the sessionstate mode
            // is set to InProc in the Web.config file. If session mode is set to StateServer 
            // or SQLServer, the event is not raised.
        }
    }
}
